@@ -1,11 +1,11 @@
 # $File: //depot/OurNet-BBS/BBS/CVIC/Article.pm $ $Author: autrijus $
-# $Revision: #2 $ $Change: 1134 $ $DateTime: 2001/06/14 18:08:06 $
+# $Revision: #3 $ $Change: 1525 $ $DateTime: 2001/08/17 22:49:33 $
 
 package OurNet::BBS::CVIC::Article;
 
 use base qw/OurNet::BBS::MAPLE2::Article/;
-use fields qw/_cache/;
+use fields qw/_ego _hash/;
 
-BEGIN {__PACKAGE__->initvars()};
+use OurNet::BBS::Base;
 
 1;
