@@ -2,8 +2,8 @@ package OurNet::BBS::CVIC::User;
 $VERSION = "0.1";
 
 use base qw/OurNet::BBS::MAPLE2::User/;
-use fields qw/_cache/;
+use fields qw/_ego _hash/;
 
-BEGIN {__PACKAGE__->initvars()};
+use OurNet::BBS::Base;;
 
 1;
